using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Net;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;

namespace CogArtTech.Utilities
{
    /// <summary>
    /// This class encapsulates translation services provided by AltaVista's BabelFish site.
    /// 
    /// This code is adapted from code originally provided by Mathew Brealy in his article,
    /// Translation Service in C#. This article can be found at:
    ///http://www.codeproject.com/cs-webservices/translation.asp?df=100&amp;forumid=31003&amp;exp=0&amp;select=1363494
    /// 
    /// </summary>
    /// <remarks>
    /// Also, when you use this code, make sure that you add the following entry into the configuration section
    /// of the web.config file:
    /// 
    ///<c>&lt;system.net&gt;</c>
    ///
    ///<c>&lt;settings&gt;</c>
    /// 
    ///<c>&lt;httpWebRequest useUnsafeHeaderParsing="true" /&gt;</c>
    ///<c>&lt;/settings&gt;</c>
    ///<c>&lt;/system.net&gt; . </c>
    /// 
    /// 
    /// You need this entry in the web.config file so that the response string returned by the call to the 
    /// BabelFish site parses without incident.
    ///
    /// </remarks>

    public class Translator
    {
        readonly string[] ValidTranslationModes = new string[] { "en_zh", "en_fr", "en_de", "en_it", "en_ja", "en_ko", "en_pt", "en_es" };
        const string BABELFISHURL = "http://babelfish.altavista.com/babelfish/tr";
        const string BABELFISHREFERER = "http://babelfish.altavista.com/";
        Page _Page = null;


        /// <summary>
        /// Translates the specified translation mode.
        /// </summary>
        /// <remarks>This method makes a behind the scenes HTTP call to the BabelFish server.</remarks>
        /// <param name="TranslationMode">The translation mode</param>
        /// <param name="SourceData">The source data, that data that you want to translate</param>
        /// <returns>A translated string</returns>
        /// <exception cref="TranslatorException">Thrown on BabelFish configuration problems. Otherwise, exceptions are of the usual CLR variety.</exception>
        /// 
        public string Translate(Page ClientPage, string SourceData)
        {
            _Page = ClientPage;
            // validate and remove trailing spaces
            if (ClientPage == null) throw new ArgumentException("You have passed a null ClientPage. ClientPage must be a valid Web Page");
            if (SourceData == null) throw new ArgumentNullException("SourceData");
            string TranslationMode = GetTranslationMode().Trim();
            if (TranslationMode.Length > 0)
            {
                SourceData = SourceData.Trim();
                //SourceData = "Good Morning";
                // check for valid TranslationModes
                bool validTranslationMode = false;
                for (int i = 0; i < ValidTranslationModes.Length; i++)
                {
                    if (ValidTranslationModes[i] == TranslationMode)
                    {
                        validTranslationMode = true;
                        break;
                    }
                }
                if (!validTranslationMode) throw new TranslatorException("The TranslationMode specified was not a valid translation TranslationMode");
                Uri uri = new Uri(BABELFISHURL);
                HttpWebRequest request = (HttpWebRequest)WebRequest.Create(uri);
                request.Referer = BABELFISHREFERER;
                // Encode all the SourceData 
                string postSourceData;
                postSourceData = "lp=" + TranslationMode + "&tt=urltext&intl=1&doit=done&urltext=" + HttpUtility.UrlEncode(SourceData);
                request.Method = "POST";
                request.ContentType = "application/x-www-form-urlencoded";
                request.ContentLength = postSourceData.Length;
                request.UserAgent = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)";

                Stream writeStream = request.GetRequestStream();
                UTF8Encoding encoding = new UTF8Encoding();
                byte[] bytes = encoding.GetBytes(postSourceData);
                writeStream.Write(bytes, 0, bytes.Length);
                writeStream.Close();
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();
                Stream responseStream = response.GetResponseStream();
                StreamReader readStream = new StreamReader(responseStream, Encoding.UTF8);
                string page = readStream.ReadToEnd();
                Regex reg = new Regex(@"<div style=padding:10px;>((?:.|\n)*?)</div>", RegexOptions.IgnoreCase);
                MatchCollection matches = reg.Matches(page);
                if (matches.Count != 1 || matches[0].Groups.Count != 2)
                {
                    string msg = "The HTML returned from Babelfish appears to have changed. Please check for an updated regular expression";
                    throw new TranslatorException(msg);
                }
                return matches[0].Groups[1].Value;
            }

            return SourceData;
        }

        /// <summary>
        /// Gets the translation code string, as defined by the containing page Culture <see cref="Page.Culture"/>attribute.
        /// </summary>
        /// <returns>A string that is a compatible TranslationMode value as defined by a predefined Property
        /// value from the class, <see cref="CogArtTech.Utilities.TranslationKey"/>. TranslationMode is a parameter in 
        /// of the <see cref="CogArtTech.Utilities.Translator"/>.Translate() method.
        /// </returns>
        private string GetTranslationMode()
        {
            TranslationKey tkey = new TranslationKey();
            if (_Page.Culture.ToUpper().IndexOf("FRENCH") != -1 || _Page.Request.UserLanguages[0].IndexOf("fr-") != -1) return tkey.EnglishToFrench;
            if (_Page.Culture.ToUpper().IndexOf("GERMAN") != -1 || _Page.Request.UserLanguages[0].IndexOf("de-") != -1) return tkey.EnglishToGerman;
            if (_Page.Culture.ToUpper().IndexOf("ITALIAN") != -1 || _Page.Request.UserLanguages[0].IndexOf("it-") != -1) return tkey.EnglishToItalian;
            if (_Page.Culture.ToUpper().IndexOf("SPANISH") != -1 || _Page.Request.UserLanguages[0].IndexOf("es-") != -1) return tkey.EnglishToSpanish;
            if (_Page.Culture.ToUpper().IndexOf("JAPANESE") != -1 || _Page.Request.UserLanguages[0].IndexOf("jp-") != -1) return tkey.EnglishToJapanese;
            if (_Page.Culture.ToUpper().IndexOf("PORTUGESE") != -1 || _Page.Request.UserLanguages[0].IndexOf("pt-") != -1) return tkey.EnglishToPortuguese;
            if (_Page.Culture.ToUpper().IndexOf("KOREAN") != -1 || _Page.Request.UserLanguages[0].IndexOf("ko-") != -1) return tkey.EnglishToKorean;
            if (_Page.Culture.ToUpper().IndexOf("CHINESE") != -1 || _Page.Request.UserLanguages[0].IndexOf("zh-") != -1) return tkey.EnglishToChineseSimple;

            return string.Empty;        }
    }
}
